/*
* Project name:
    SET_NAME_BT (Utility to set the name of Bluetooth module)
* Copyright:
    (c) RoboTech srl, 2007.
* Description:
    Demonstration of an LCD remote control application
    using the Bluetooth serial adapter board.
* Test configuration:
    MCU:             PIC16F877A
    Dev.Board:       EasyPIC3
    Oscillator:      HS, 08.0000 MHz
    Ext. Modules:    LCD 2x16
    SW:              mikroC v6.0
* NOTES:
    None.
*/

#define BT_MAX_BUFFER 16

char BT_Buffer[BT_MAX_BUFFER]; // buffer for the received text

int timeout = 0;
char BT_Receive = 1;

int BT_Length;
char BT_Counter = 0, BT_Packet, BT_Opcode, BT_Status;

void BT_ParsePacket(char bufsize);
void BT_ReadResponse();

void BT_SetEventsDefault();
void BT_ResetFirmware();
void BT_ReadName();
void BT_SetName_RBT001();
void BT_SetName_EasyBT();

#define TIMEOUT_1S 122

enum BTS
{
	// generic status
	BTS_NONE,							// no-op status
	BTS_TIMEOUT,						// receive timed out
	BTS_INVALID,						// invalid packet
	BTS_CHECKSUM,						// checksum invalid
	BTS_UNKNOWN,						// packet valid, but unknown

	// replies
	BTS_READY,							// firmware ready
	BTS_EVENTS_OK, BTS_EVENTS_ERR,		// set event mask status
	BTS_NAME_OK, BTS_NAME_ERR,			// set local name status
  BTS_NAME_READ, BTS_NAME_FAIL,   // get local name status
  
	BTS_MAX								// count (insert values before)
};

void Show_Choice();
void Show_Error();
void Show_Name();

void main()
{
	LCD_Init(&PORTD);
	USART_Init(9600);

	OPTION_REG = 0x87; // Timer0 clock = cpu/256

	LCD_Cmd(LCD_CLEAR);
	LCD_Cmd(LCD_CURSOR_OFF);
	LCD_Out(1,1,"BT Module ...");

	Delay_ms(1000); // allow BT to send ready message
	BT_SetEventsDefault(); // restore default BT notifications
	BT_ReadResponse();
	if (BT_Status != BTS_EVENTS_OK)
	{
		Show_Error();
		while (1);
	}

	BT_ResetFirmware();
	BT_ReadResponse();
	if (BT_Status != BTS_READY)
	{
		Show_Error();
		while (1);
	}

	Show_Choice();

	while (1)
	{
		if (Button(&PORTB, 0, 5, 1))
		{
			LCD_Out(1,1,"Set \"RBT-001\"...");
			while (Button(&PORTB, 0, 5, 1));
			BT_SetName_RBT001();
			BT_ReadResponse();
			if (BT_Status != BTS_NAME_OK)
			{
				Show_Error();
				Delay_ms(3000);
				Show_Choice();
				continue;
			}
			break;
		}
		if (Button(&PORTB, 1, 5, 1))
		{
			LCD_Out(1,1,"Set \"EasyBT\"...");
			while (Button(&PORTB, 1, 5, 1));
			BT_SetName_EasyBT();
			BT_ReadResponse();
			if (BT_Status != BTS_NAME_OK)
			{
				Show_Error();
				Delay_ms(3000);
				Show_Choice();
				continue;
			}
			break;
		}
	}
	LCD_Cmd(LCD_CLEAR);
	LCD_Out(1,1,"BT Reset ...");

	BT_ResetFirmware();
	BT_ReadResponse();
	if (BT_Status != BTS_READY)
	{
		Show_Error();
		while (1);
	}

	LCD_Out(2,1,"OK");
  Delay_ms(3000);
	LCD_Out(1,1,"BT Name ...");

	BT_ReadName();
	BT_ReadResponse();
	if (BT_Status != BTS_NAME_READ)
	{
		Show_Error();
		while (1);
	}
	Show_Name();
}

void Show_Error()
{
  LCD_Out(2,1,"Error           ");
  ByteToStr(BT_Status, BT_Buffer);
  LCD_Out(2,7,BT_Buffer);
  //ByteToStr(BT_Counter, BT_Buffer);
  //LCD_Out(2,11,BT_Buffer);
}

void Show_Name()
{
	BT_Length = BT_Buffer[1];
	BT_Buffer[BT_Length+2] = 0;
	BT_Buffer[BT_Length+1] = '\"';
	BT_Buffer[1] = '\"';
	LCD_Out(2,1,BT_Buffer+1);
}

void Show_Choice()
{
	LCD_Cmd(LCD_CLEAR);
	LCD_Out(1,1,"RB1 -> \"EasyBT\"");
	LCD_Out(2,1,"RB0 -> \"RBT-001\"");
}

void interrupt()
{
	++timeout;
	if (timeout == TIMEOUT_1S)
		BT_Receive = 0;

	TMR0 = 0;
	INTCON = 0xA0;
}

void BT_ReadResponse()
{
	char rx, chksum;
	int i;

	BT_Receive = 1;
	BT_Status = BTS_TIMEOUT;

	timeout = 0;
	TMR0 = 0;
	INTCON = 0xA0;       // Enable Timer0

	for (BT_Counter = 0; BT_Receive != 0; ++BT_Counter)
	{
		while (BT_Receive && !USART_Data_Ready());
		if (!BT_Receive) break;
		
		rx = USART_Read(); // char count

		switch (BT_Counter)
		{
		case 0:
			if (rx != 0x02)
			{
				BT_Status = BTS_INVALID;
				BT_Receive = 0;
				break;
			}
			break;
		case 1:
			BT_Packet = rx;
			if (rx != 0x43 && rx != 0x69)
			{
				BT_Status = BTS_INVALID;
				BT_Receive = 0;
				break;
			}
			chksum = rx;
			break;
		case 2:
			BT_Opcode = rx;
			chksum += rx;
			break;
		case 3:
			BT_Length = rx;
			chksum += rx;
			break;
		case 4:
			BT_Length |= rx << 8;
			chksum += rx;
			break;
		case 5:
			if (chksum != rx)
			{
				BT_Status = BTS_CHECKSUM;
				BT_Receive = 0;
				break;
			}
			i = 0;
			break;
		case 6:
      --BT_Counter;
			if (BT_Length > 0)
			{
				--BT_Length;
				if (i < BT_MAX_BUFFER)
					BT_Buffer[i++] = rx;
			}
			else
			{
				if (rx != 0x03)
				{
					BT_Status = BTS_INVALID;
					BT_Receive = 0;
					break;
				}
				// Disable timer
				TMR0 = 0;
				INTCON = 0x00;
				
				BT_ParsePacket(i);
				return;
			}
		}
	}

	// Disable timer
	TMR0 = 0;
	INTCON = 0x00;
}

void BT_ParsePacket(char bufsize)
{
	BT_Status = BTS_INVALID;
	if (BT_Packet == 0x69) // IND
	{
		BT_Status = BTS_UNKNOWN;
		if (BT_Opcode == 0x25)
			BT_Status = BTS_READY;
	}
	else if (BT_Packet == 0x43) // CFM
	{
		BT_Status = BTS_UNKNOWN;
		if (BT_Opcode == 0x4E)
		{
			BT_Status = BTS_EVENTS_ERR;
			if (BT_Buffer[0] == 0)
				BT_Status = BTS_EVENTS_OK;
		}
		else if (BT_Opcode == 0x04)
		{
				BT_Status = BTS_NAME_ERR;
				if (BT_Buffer[0] == 0)
					BT_Status = BTS_NAME_OK;
		}
		else if (BT_Opcode == 0x03)
		{
				BT_Status = BTS_NAME_FAIL;
				if (BT_Buffer[0] == 0)
					BT_Status = BTS_NAME_READ;
		}
	}
}

void BT_SetEventsDefault()
{
	USART_Write(0x02);
	USART_Write(0x52);
	USART_Write(0x4E);
	USART_Write(0x01);
	USART_Write(0x00);
	USART_Write(0xA1);
	USART_Write(0x01);
	USART_Write(0x03);
}

void BT_ResetFirmware()
{
	USART_Write(0x02);
	USART_Write(0x52);
	USART_Write(0x26);
	USART_Write(0x00);
	USART_Write(0x00);
	USART_Write(0x78);
	USART_Write(0x03);
}

void BT_ReadName()
{
	USART_Write(0x02);
	USART_Write(0x52);
	USART_Write(0x03);
	USART_Write(0x00);
	USART_Write(0x00);
	USART_Write(0x55);
	USART_Write(0x03);
}

void BT_SetName_RBT001()
{
	USART_Write(0x02);
	USART_Write(0x52);
	USART_Write(0x04);
	USART_Write(0x09);
	USART_Write(0x00);
	USART_Write(0x5F);
	USART_Write(0x08);
	USART_Write(0x52);
	USART_Write(0x42);
	USART_Write(0x54);
	USART_Write(0x2D);
	USART_Write(0x30);
	USART_Write(0x30);
	USART_Write(0x31);
	USART_Write(0x00);
	USART_Write(0x03);
}

void BT_SetName_EasyBT()
{
	USART_Write(0x02);
	USART_Write(0x52);
	USART_Write(0x04);
	USART_Write(0x08);
	USART_Write(0x00);
	USART_Write(0x5E);
	USART_Write(0x07);
	USART_Write(0x45);
	USART_Write(0x61);
	USART_Write(0x73);
	USART_Write(0x79);
	USART_Write(0x42);
	USART_Write(0x54);
	USART_Write(0x00);
	USART_Write(0x03);
}
